	/*

	Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#ifndef _SYNTH_IMPL_H
#define _SYNTH_IMPL_H

#include "synth.h"
#include "synthmodule.h"
#include "moduleserver.h"
#include "artsobject.h"
#include "cache.h"
#include "midi.h"

#ifdef HAVE_MINI_STL
#include <ministl/list>
#include <ministl/string>
#else
#include <list>
#include <string>
#endif

class ArtsIOEvent;

class Synthesizer_impl :public ArtsObject_impl,
						public Arts::Synthesizer_skel
						 
{
	Cache *_Cache;
	MidiChannel_impl *_MidiChannel;

	SynthModule	**SynthModules;
	int MCount;

	bool inScheduler;

	list<ArtsIOEvent *> IOEvents;
	long _StructChangeCount;

	list<long> killList;

/** status watching -> don't bring down the system by hogging the cpu **/

	long status_stalled;
	long status_clocks;
	long status_cycles;
	bool status_halted;
	float status_cpu_usage;
	long status_cache_usage;

protected:
	ModuleServer<SynthModule> *MS_SynthModule;
	float *AllocArg(string name, int flags);

	long nextID;		// execution IDs

	Arts::RealtimeConfig _RTConfig;
	Arts::StructureDescSeq _Structures;

public:
	long StructChangeCount();

	void Run();
	bool Exec();
	void kill(long ID);
	Cache *getCache();
	MidiChannel_impl *getMidiChannel();

	void removeDynamicConnections (SynthModule *module);
	void addDynamicConnection (SynthModule *target, SynthModule *source, long unsigned int size);

	long createConnectedStructure(Arts::StructureDesc *structuredesc,
											list<SynthModule *> interfaces);

	Synthesizer_impl();
	~Synthesizer_impl();
	void cleanUp();

// exported CORBA attributes

	Arts::RealtimeConfig RTConfig();
	void RTConfig(const Arts::RealtimeConfig &newconfig);

	Arts::ResourceInfoSeq *Resources();

	Arts::StructureDescSeq *publishedStructures();

// exported CORBA functions

    CORBA::Long getModuleCount();
    CORBA::Boolean getModuleInfo( CORBA::Long nr, Arts::ModuleInfo*& info );

    void Reset();
    Arts::Status getStatus();

    Arts::StructureDesc *createStructureDesc();
    void publishStructureDesc(Arts::StructureDesc *structuredesc);
    Arts::StructureDesc *lookupStructureDesc(const char *name);
 
	CORBA::Long createStructure(Arts::StructureDesc *structuredesc);
    CORBA::Boolean isExecuting(long int id);
    CORBA::Boolean freeStructure(long int id);
};

#endif
